/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.action;

import com.google.common.base.Strings;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import org.opengroup.osdu.register.model.ParsedAction;
import org.opengroup.osdu.register.model.TestPayload;
import org.opengroup.osdu.register.model.ValidationErrorResponse;
import org.opengroup.osdu.register.util.TestBase;
import org.opengroup.osdu.register.util.TestPayloadReader;
import org.opengroup.osdu.register.util.TestUtils;
import com.sun.jersey.api.client.ClientResponse;
import org.apache.http.HttpStatus;
import org.junit.Test;

import javax.ws.rs.HttpMethod;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static org.hamcrest.collection.IsIterableContainingInAnyOrder.containsInAnyOrder;
import static org.junit.Assert.*;

public abstract class TestActionApiTest extends TestBase {

    private static final String PATH = "api/register/v1/action:test";

    private TestPayloadReader reader = new TestPayloadReader();

    @Test
    public void should_return20X_when_usingCredentialsWithPermission() throws Exception {
        String validPayload = reader.getTestPayload("actionApi/validTestAction.json");

        ClientResponse response = TestUtils.send(PATH, HttpMethod.POST, testUtils.getOpsAccessToken(), validPayload, "");
        assertEquals(HttpStatus.SC_OK, response.getStatus());

        String body = response.getEntity(String.class);
        ParsedAction parsedAction = new Gson().fromJson(body, ParsedAction.class);
        assertEquals("https://myapp.osdu.opengroup.org/action/abc123", parsedAction.url);
        assertTrue(Strings.isNullOrEmpty(parsedAction.errors));
    }

    @Test
    public void should_return40XResponseCode_when_makingRequest_withInvalidPayload() throws Exception {
        List<TestPayload> payloadList = new ArrayList<>();
        payloadList.add(TestPayload.builder()
                .expectedErrors(Arrays.asList("action.url must be a valid URL", "testPayload must not be null"))
                .payloadFileName("actionApi/invalidUrlTestActionPayload.json").build());
        payloadList.add(TestPayload.builder()
                .expectedErrors(Arrays.asList("action.url must be a valid URL"))
                .payloadFileName("actionApi/nonHttpsTestAction.json").build());
        payloadList.add(TestPayload.builder()
                .expectedErrors(Arrays.asList("action.filter Found empty filter values. Must provide a filter value. Use '*' if you want to match all values for a filter property."))
                .payloadFileName("actionApi/invalidFilterEntityTypeTestAction.json").build());
        payloadList.add(TestPayload.builder()
                .expectedErrors(Arrays.asList("action.name must match \"^[A-Za-z0-9- ]{2,50}\"", "action.name must not be empty"))
                .payloadFileName("actionApi/invalidNameTestAction.json").build());

        for (TestPayload payload : payloadList) {
            makeRequestWithInvalidPayload(payload.getExpectedErrors(), payload.getPayloadFileName());
        }
    }

    public void makeRequestWithInvalidPayload(List<String> expectedErrors, String payloadFileName) throws Exception {
        String payload = reader.getTestPayload(payloadFileName);

        ClientResponse response = TestUtils.send(PATH, HttpMethod.POST, testUtils.getOpsAccessToken(), payload, "");
        assertEquals(HttpStatus.SC_BAD_REQUEST, response.getStatus());
        String body = response.getEntity(String.class);
        ValidationErrorResponse validationErrorResponse = new Gson().fromJson(body, ValidationErrorResponse.class);
        List<String> actualErrors = new Gson().fromJson(validationErrorResponse.getMessage(), new TypeToken<List<String>>() {
        }.getType());
        assertThat(expectedErrors, containsInAnyOrder(actualErrors.toArray()));
    }
}
